@@ -76,7 +76,7 @@ in the file.
 
 =head1 VERSION
 
-0.034
+0.045
 
 =head1 AUTHOR
 
@@ -100,12 +100,11 @@ use Exporter;
 @ISA = qw(Exporter);
 
 @EXPORT = qw( DEBUG FIXPROBLEMS Error Warning NullError );
-@EXPORT_OK = qw( DEBUG FIXPROBLEMS Error Warning NullError );
 
 # ##############
 # General things
 
-$VERSION = "0.034";
+$VERSION = "0.045";
 
 # Sets the debug level
 $DEBUG = 0;
@@ -118,7 +117,7 @@ sub FIXPROBLEMS () { $FIXPROBLEMS }
 
 # Holds the text of the error, if there was one
 $errstr = '';
-sub errstr ()	{ $errstr }
+sub errstr ()	{ (ref $_[0] ? $_[0]->{'errstr'} : $errstr); }
 
 # Issues warning to STDERR if there is debug level set, but does Error
 # if not FIXPROBLEMS
@@ -131,13 +130,15 @@ sub Warning (@)
 # Prints error on STDERR if there is debug level set and sets $errstr
 sub Error (@)
 	{
-	shift if ref $_[0];
+	my $self;
+	$self = shift if ref $_[0];
 	print STDERR @_ if DEBUG;
-	$errstr .= join '', @_;
+	(defined $self ? $self->{'errstr'} : $errstr) .= join '', @_;
 	}
 # Nulls the $errstr, should be used in methods called from the mail
 # program
-sub NullError ()	{ $errstr = ''; }
+sub NullError
+	{ if (ref $_[0]) { $_[0]->{'errstr'} = ''; } else { $errstr = ''; } }
 
 
 # Contructor. If it is passed a name of the file, it opens it and
@@ -148,7 +149,7 @@ sub new
 	my $class = shift;
 	my $new = {};
 	bless $new, $class;
-	if (@_)	{ $new->open(shift) and return $new; return; }
+	if (@_)	{ $new->open(@_) and return $new; return; }
 	return $new;
 	}
 # Open the file. This is the second chance when filename can be
@@ -172,6 +173,7 @@ sub open
 	$fh->open($filename, $mode) or do
 		{ Error "Error opening file $filename: $!\n"; return; };
 				# open the file
+	binmode($fh);		# f..k Windoze
 
 	my $perms = (stat($filename))[2] & 0777;
 
@@ -181,7 +183,7 @@ sub open
 	unless ($self->can('read_header'))
 		{ Error "Method read_header not defined for $self\n"; return; }
 	
-	unless ($self->read_header())	# read_header should be
+	unless ($self->read_header(@_))	# read_header should be
 		{ return; }		# defined in the derived class
 
 	1;
@@ -189,10 +191,10 @@ sub open
 # Closes the file
 sub close
 	{
-	NullError();
 	my $self = shift;
+	$self->NullError();
 	if (not defined $self->{'opened'})
-		{ Error "Can't close file that is not opened\n"; return; }
+		{ $self->Error("Can't close file that is not opened\n"); return; }
 	$self->{'fh'}->close();
 	delete @{$self}{'opened', 'fh'};
 	1;
@@ -200,16 +202,16 @@ sub close
 # Drop (unlink) the file
 sub drop
 	{
-	NullError();
 	my $self = shift;
 	my $filename = $self;
+	$self->NullError();
 	if (ref $self)
 		{
 		$filename = $self->{'filename'};
-		$self->close() if ($self->{'opened'});
+		$self->close() if defined $self->{'opened'};
 		}
 	unlink $filename or
-		do { Error "Error unlinking file $filename: $!\n"; return; };
+		do { $self->Error("Error unlinking file $filename: $!\n"); return; };
 	1;	
 	}
 # Create new file
@@ -232,6 +234,7 @@ sub create_file
 		{
 		@{$self}{ qw( fh filename perms writable opened ) }
 			= ( $fh, $name, $perms, 1, 1 );
+		binmode($fh);		# f..k Windoze	
 		return $self;
 		};
 	return;
@@ -247,13 +250,13 @@ sub get_record_offset
 	my ($header_len, $record_len) = @{$self}{ qw( header_len record_len ) };
 	unless (defined $header_len and defined $record_len)
 		{
-		Error "Header and record lengths not known in get_record_offset\n";
+		$self->Error("Header and record lengths not known in get_record_offset\n");
 		return;
 		}
 	my $num = shift;
 	unless (defined $num)
 		{
-		Error "Number of the record must be specified in get_record_offset\n";
+		$self->Error("Number of the record must be specified in get_record_offset\n");
 		return;
 		}
 	return $header_len + $num * $record_len;	
@@ -276,14 +279,14 @@ sub seek_to
 
 			# the file should really be opened and writable
 	if (not defined $self->{'opened'})
-		{ Error "The file $filename is not opened\n"; return; }
-	if (not $self->{'writable'})
-		{ Error "The file $filename is not writable\n"; return; }
+		{ $self->Error("The file $filename is not opened\n"); return; }
+	### if (not $self->{'writable'})
+	###	{ $self->Error("The file $filename is not writable\n"); return; }
 
 	delete $self->{'tell'};	# we cancel the tell position
 
 	$self->{'fh'}->seek($offset, 0)	# seek to the offset
-		or do { Error "Error seeking to offset $offset on $filename: $!\n";
+		or do { $self->Error("Error seeking to offset $offset on $filename: $!\n");
 		return;
 		};
 	1;
@@ -295,10 +298,11 @@ sub seek_to
 sub read_record
 	{
 	my ($self, $num, $in_length) = @_;
+
 	if (not defined $num)
-		{ Error "Record number to read must be specified\n"; return; }
+		{ $self->Error("Record number to read must be specified\n"); return; }
 	if ($num > $self->last_record())
-		{ Error "Can't read record $num, there is not so many of them\n"; return; }
+		{ $self->Error("Can't read record $num, there is not so many of them\n"); return; }
 
 	if (defined $self->{'cached_num'} and $num == $self->{'cached_num'})
 		{
@@ -313,15 +317,18 @@ sub read_record
 	my ($fh, $record_len) = @{$self}{ qw( fh record_len ) };
 	my $buffer;
 
-	my $length = $record_len if ((not defined $in_length) or $in_length == -1);
-	my $readlen = $fh->read($buffer, $length);
+	$in_length = $record_len unless defined $in_length;
 
-	if ((not defined $in_length or $in_length != -1) and $readlen != $length)
+	my $actually_read = $fh->read($buffer, ($in_length == -1 ?
+						$record_len : $in_length));
+	
+	if ($actually_read != $in_length and $in_length != -1)
 		{
-		Warning "Error reading the whole record num $num\n";
+		$self->Warning("Error reading the whole record num $num\n");
 		return unless FIXPROBLEMS;
 		};
-	$self->{'tell'} = (defined $tell) ? $tell + $length : $fh->tell();
+	
+	$self->{'tell'} = (defined $tell) ? $tell + $actually_read : $fh->tell();
 	
 	$self->{'cached_num'} = $num;
 	if (defined $self->{'unpack_template'})
@@ -343,8 +350,10 @@ sub read_record
 sub write_record
 	{
 	my ($self, $num) = (shift, shift);
+	if (not $self->{'writable'})
+		{ $self->Error("The file $self->{'filename'} is not writable\n"); return; }
 	if (not defined $num)
-		{ Error "Record number to write must be specified\n"; return; }
+		{ $self->Error("Record number to write must be specified\n"); return; }
 	if (defined $self->{'cached_num'} and $num == $self->{'cached_num'})
 		{ delete $self->{'cached_num'}; }
 	$self->seek_to_record($num) or return;
@@ -353,7 +362,7 @@ sub write_record
 	local ($,, $\) = ("", "");
 	my $fh = $self->{'fh'};
 	$fh->print(@_) or
-		do { Error "Error writing record $num: $!\n"; return; } ;
+		do { $self->Error("Error writing record $num: $!\n"); return; } ;
 	( $num == 0 ) ? "0E0" : $num;
 	}
 
@@ -361,14 +370,15 @@ sub write_record
 sub write_to
 	{
 	my ($self, $offset) = (shift, shift);
-	
+	if (not $self->{'writable'})
+		{ $self->Error("The file $self->{'filename'} is not writable\n"); return; }
 	$self->seek_to($offset) or return;
 	delete $self->{'tell'};
 	
 	local ($,, $\) = ("", "");
 	my $fh = $self->{'fh'};
 	$fh->print(@_) or
-		do { Error "Error writing at offset $offset: $!\n"; return; } ;
+		do { $self->Error("Error writing at offset $offset: $!\n"); return; } ;
 	( $offset == 0 ) ? "0E0" : $offset;
 	}
 
